package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.random.RandomGenerator;

public record MixtureProbabilityDensityFunction(
        @NotNull ProbabilityDensityFunction a,
        @NotNull ProbabilityDensityFunction b,
        double weight
) implements ProbabilityDensityFunction {
    public MixtureProbabilityDensityFunction(@NotNull ProbabilityDensityFunction a, @NotNull ProbabilityDensityFunction b) {
        this(a, b, 0.5);
    }

    public MixtureProbabilityDensityFunction {
        Objects.requireNonNull(a);
        Objects.requireNonNull(b);
        weight = Math.clamp(weight, 0, 1);
    }

    @Override
    public double value(@NotNull Vec3 direction) {
        return weight * a.value(direction) + (1 - weight) * b.value(direction);
    }

    @Override
    public @NotNull Vec3 generate(@NotNull RandomGenerator random) {
        if (random.nextDouble() < weight) {
            return a.generate(random);
        } else {
            return b.generate(random);
        }
    }
}
