package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.scene.Target;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.random.RandomGenerator;

/**
 * A probability density function targeting a target.
 */
public record TargetingProbabilityDensityFunction(@NotNull Vec3 origin, @NotNull Target target) implements ProbabilityDensityFunction {
    public TargetingProbabilityDensityFunction {
        Objects.requireNonNull(origin, "origin");
        Objects.requireNonNull(target, "target");
    }

    @Override
    public double value(@NotNull Vec3 direction) {
        return target.getProbabilityDensity(origin, direction);
    }

    @Override
    public @NotNull Vec3 generate(@NotNull RandomGenerator random) {
        return target.getTargetingDirection(origin, random);
    }
}
